import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["visibility"];
import React from 'react';
import ReactDOM from 'react-dom';
import { useSpring, animated } from '@react-spring/web';
import { useAnchor } from '../hooks/useAnchor';
import useLatestWhen from '../hooks/useLatestWhen';
import usePortalElement from '../hooks/usePortalElement';
import usePrevious from '../hooks/usePrevious';
//
import useChartContext from '../utils/chartContext';
import TooltipRenderer from './TooltipRenderer';
import useRect from '../hooks/useRect'; //

function defaultTooltip(options) {
  var _options$align, _options$alignPriorit, _options$padding, _options$tooltipArrow, _options$render;

  if (options === void 0) {
    options = {};
  }

  return _extends({}, options, {
    align: (_options$align = options.align) != null ? _options$align : 'auto',
    alignPriority: (_options$alignPriorit = options.alignPriority) != null ? _options$alignPriorit : ['right', 'topRight', 'bottomRight', 'left', 'topLeft', 'bottomLeft', 'top', 'bottom'],
    padding: (_options$padding = options.padding) != null ? _options$padding : 5,
    tooltipArrowPadding: (_options$tooltipArrow = options.tooltipArrowPadding) != null ? _options$tooltipArrow : 7,
    // anchor: options.anchor ?? 'closest',
    render: (_options$render = options.render) != null ? _options$render : TooltipRenderer
  });
}

export default function Tooltip() {
  var _getOptions$tooltip, _secondaryAxes$find, _useLatestWhen;

  var _useChartContext = useChartContext(),
      useFocusedDatumAtom = _useChartContext.useFocusedDatumAtom,
      getOptions = _useChartContext.getOptions,
      primaryAxis = _useChartContext.primaryAxis,
      secondaryAxes = _useChartContext.secondaryAxes,
      getDatumStatusStyle = _useChartContext.getDatumStatusStyle,
      svgRef = _useChartContext.svgRef;

  var _useFocusedDatumAtom = useFocusedDatumAtom(),
      focusedDatum = _useFocusedDatumAtom[0];

  var latestFocusedDatum = useLatestWhen(focusedDatum, !!focusedDatum);
  var preTooltipOptions = (_getOptions$tooltip = getOptions().tooltip) != null ? _getOptions$tooltip : true;
  var secondaryAxis = (_secondaryAxes$find = secondaryAxes.find(function (d) {
    return d.id === (latestFocusedDatum == null ? void 0 : latestFocusedDatum.secondaryAxisId);
  })) != null ? _secondaryAxes$find : secondaryAxes[0];
  var tooltipOptions = React.useMemo(function () {
    return defaultTooltip(typeof preTooltipOptions === 'boolean' ? {} : preTooltipOptions);
  }, [preTooltipOptions]);
  var portalEl = usePortalElement();

  var _React$useState = React.useState(),
      tooltipEl = _React$useState[0],
      setTooltipEl = _React$useState[1];

  var svgRect = useRect(svgRef.current, !!(focusedDatum != null && focusedDatum.element));
  var anchorEl = React.useMemo(function () {
    var _latestFocusedDatum$e, _latestFocusedDatum$e2, _anchorRect$left, _anchorRect$top, _anchorRect$width, _anchorRect$height;

    var anchorRect = (_latestFocusedDatum$e = latestFocusedDatum == null ? void 0 : (_latestFocusedDatum$e2 = latestFocusedDatum.element) == null ? void 0 : _latestFocusedDatum$e2.getBoundingClientRect()) != null ? _latestFocusedDatum$e : null;

    if (!anchorRect) {
      return null;
    }

    if (!svgRect) return;
    var translateX = (_anchorRect$left = anchorRect.left) != null ? _anchorRect$left : 0;
    var translateY = (_anchorRect$top = anchorRect.top) != null ? _anchorRect$top : 0;
    var width = (_anchorRect$width = anchorRect.width) != null ? _anchorRect$width : 0;
    var height = (_anchorRect$height = anchorRect.height) != null ? _anchorRect$height : 0;
    var box = {
      x: translateY,
      y: translateX,
      top: translateY,
      left: translateX,
      bottom: translateY + width,
      right: translateX + height,
      width: width,
      height: height,
      toJSON: function toJSON() {
        return {};
      }
    };

    box.toJSON = function () {
      return box;
    };

    return {
      getBoundingClientRect: function getBoundingClientRect() {
        return box;
      }
    };
  }, [latestFocusedDatum == null ? void 0 : latestFocusedDatum.element, svgRect]);
  var anchor = useAnchor({
    show: !!focusedDatum,
    portalEl: portalEl,
    anchorEl: anchorEl,
    tooltipEl: tooltipEl,
    side: ['right', 'left', 'top', 'bottom']
  });
  var previousAnchor = usePrevious(anchor);
  var latestStableAnchor = (_useLatestWhen = useLatestWhen(anchor, !!anchor.fit)) != null ? _useLatestWhen : anchor;

  var _latestStableAnchor$s = latestStableAnchor.style,
      visibility = _latestStableAnchor$s.visibility,
      anchorStyle = _objectWithoutPropertiesLoose(_latestStableAnchor$s, _excluded);

  var springProps = useSpring(_extends({}, anchorStyle, {
    left: anchorStyle.left || 0,
    top: anchorStyle.top || 0,
    opacity: !!focusedDatum ? 1 : 0,
    config: {
      mass: 1,
      tension: 210,
      friction: 30
    },
    immediate: function immediate(key) {
      if (['left', 'top'].includes(key)) {
        return Number.isNaN(previousAnchor == null ? void 0 : previousAnchor.style.left);
      }

      return false;
    }
  }));
  var show = !!preTooltipOptions;
  var latestFit = useLatestWhen(anchor.fit, !!anchor.fit);
  return show && portalEl ? /*#__PURE__*/ReactDOM.createPortal( /*#__PURE__*/React.createElement(animated.div, {
    style: springProps
  }, /*#__PURE__*/React.createElement("div", {
    ref: function ref(el) {
      return setTooltipEl(el);
    },
    style: _extends({
      fontFamily: 'sans-serif'
    }, (latestFit == null ? void 0 : latestFit.startKey) === 'left' ? {
      padding: '0 10px'
    } : {
      padding: '10px 0'
    })
  }, tooltipOptions.render({
    getOptions: getOptions,
    focusedDatum: latestFocusedDatum,
    primaryAxis: primaryAxis,
    secondaryAxis: secondaryAxis,
    getDatumStyle: function getDatumStyle(datum) {
      return getDatumStatusStyle(datum, focusedDatum);
    },
    anchor: anchor
  }))), portalEl) : null;
}