import _extends from "@babel/runtime/helpers/esm/extends";
import { extent, max, median, min, range as d3Range } from 'd3-array';
import { scaleLinear, scaleLog, scaleTime, scaleUtc, scaleBand } from 'd3-scale';

function defaultAxisOptions(options) {
  var _options$elementType, _options$minTickPaddi, _options$tickLabelRot, _options$innerBandPad, _options$outerBandPad, _options$show, _options$stacked;

  return _extends({}, options, {
    elementType: (_options$elementType = options.elementType) != null ? _options$elementType : 'line',
    // tickCount: options.tickCount ?? 10,
    // minTickCount: options.minTickCount ?? 1,
    // maxTickCount: options.maxTickCount ?? 99999999,
    // tickSizeInner: options.tickSizeInner ?? 6,
    // tickSizeOuter: options.tickSizeOuter ?? 6,
    minTickPaddingForRotation: (_options$minTickPaddi = options.minTickPaddingForRotation) != null ? _options$minTickPaddi : 10,
    tickLabelRotationDeg: (_options$tickLabelRot = options.tickLabelRotationDeg) != null ? _options$tickLabelRot : 60,
    innerBandPadding: (_options$innerBandPad = options.innerBandPadding) != null ? _options$innerBandPad : 0.6,
    outerBandPadding: (_options$outerBandPad = options.outerBandPadding) != null ? _options$outerBandPad : 0.2,
    // showTicks: options.showTicks ?? true,
    // filterTicks: options.filterTicks ?? (d => d),
    show: (_options$show = options.show) != null ? _options$show : true,
    stacked: (_options$stacked = options.stacked) != null ? _options$stacked : false
  });
}

export default function buildAxisLinear(userOptions, series, gridDimensions, width, height) {
  var options = defaultAxisOptions(userOptions);

  if (!options.position) {
    throw new Error("Chart axes must have a valid 'position' property");
  }

  var isVertical = ['left', 'right'].indexOf(options.position) > -1; // Now we need to figure out the range

  var range = isVertical ? [gridDimensions.gridHeight, 0] : [0, gridDimensions.gridWidth];
  var outerRange = isVertical ? [height, 0] : [0, width]; // Give the scale a home

  return options.scaleType === 'time' || options.scaleType === 'localTime' ? buildTimeAxis(options, series, isVertical, range, outerRange) : options.scaleType === 'linear' || options.scaleType === 'log' ? buildLinearAxis(options, series, isVertical, range, outerRange) : options.scaleType === 'band' ? buildBandAxis(options, series, isVertical, range, outerRange) : function () {
    throw new Error('Invalid scale type');
  }(); // @ts-ignore
  // const scaleFormat = scale.tickFormat ? scale.tickFormat() : d => d
  // const userFormat = options?.format
  // const format = userFormat
  //   ? (value: unknown, index: number) =>
  //       userFormat(value, index, scaleFormat(value))
  //   : scaleFormat
  // const resolvedTickCount = options.tickCount
  // const ticks = options.filterTicks(
  //   options.tickValues ||
  //     // @ts-ignore
  //     (scale.ticks ? scale.ticks(resolvedTickCount) : scale.domain())
  // )
  // const scaleMax =
  //   options.position === 'bottom'
  //     ? -gridDimensions.gridHeight
  //     : options.position === 'left'
  //     ? gridDimensions.gridWidth
  //     : options.position === 'top'
  //     ? gridDimensions.gridHeight
  //     : -gridDimensions.gridWidth
  // // const directionMultiplier =
  // //   options.position === 'top' || options.position === 'left' ? -1 : 1
  // const transform = !isVertical ? translateX : translateY
  // const tickSpacing = Math.max(options.tickSizeInner, 0) + options.tickPadding
  // Pass down the axis config (including the scale itself) for posterity
}

function buildTimeAxis(options, series, isVertical, range, outerRange) {
  var scaleFn = options.scaleType === 'localTime' ? scaleTime : scaleUtc; // Now set the range

  var scale = scaleFn(range);
  var allDatums = series.map(function (d) {
    return d.datums;
  }).flat();

  var _extent = extent(allDatums, function (datum) {
    return options.getValue(datum.originalDatum);
  }),
      minValue = _extent[0],
      maxValue = _extent[1];

  var shouldNice = true;

  if (typeof options.min === 'number') {
    minValue = min([options.min, minValue]);
    shouldNice = false;
  }

  if (typeof options.max === 'number') {
    maxValue = max([options.max, maxValue]);
    shouldNice = false;
  }

  if (typeof options.hardMin === 'number') {
    minValue = options.hardMin;
    shouldNice = false;
  }

  if (typeof options.hardMax === 'number') {
    maxValue = options.hardMax;
    shouldNice = false;
  }

  if (minValue === undefined || maxValue === undefined) {
    console.info({
      options: options,
      series: series,
      range: range,
      values: allDatums.map(function (d) {
        return options.getValue(d.originalDatum);
      })
    });
    throw new Error('Invalid scale min/max');
  } // Set the domain


  scale.domain([minValue, maxValue]);

  if (options.invert) {
    scale.domain(Array.from(scale.domain()).reverse());
  }

  if (shouldNice) {
    scale.nice();
  }

  var outerScale = scale.copy().range(outerRange); // Supplmentary band scale

  var bandScale = options.isPrimary ? buildImpliedBandScale(options, scale, series, range) : undefined;
  var defaultFormat = scale.tickFormat();
  var formatters = {};

  var scaleFormat = function scaleFormat(value) {
    var _options$formatters$s, _options$formatters;

    return (_options$formatters$s = (_options$formatters = options.formatters) == null ? void 0 : _options$formatters.scale == null ? void 0 : _options$formatters.scale(value, _extends({}, formatters, {
      scale: undefined
    }))) != null ? _options$formatters$s : defaultFormat(value);
  };

  var tooltipFormat = function tooltipFormat(value) {
    var _options$formatters$t, _options$formatters2;

    return (_options$formatters$t = (_options$formatters2 = options.formatters) == null ? void 0 : _options$formatters2.tooltip == null ? void 0 : _options$formatters2.tooltip(value, _extends({}, formatters, {
      tooltip: undefined
    }))) != null ? _options$formatters$t : scaleFormat(value);
  };

  var cursorFormat = function cursorFormat(value) {
    var _options$formatters$c, _options$formatters3;

    return (_options$formatters$c = (_options$formatters3 = options.formatters) == null ? void 0 : _options$formatters3.cursor == null ? void 0 : _options$formatters3.cursor(value, _extends({}, formatters, {
      cursor: undefined
    }))) != null ? _options$formatters$c : tooltipFormat(value);
  };

  Object.assign(formatters, {
    "default": defaultFormat,
    scale: scaleFormat,
    tooltip: tooltipFormat,
    cursor: cursorFormat
  });
  return _extends({}, options, {
    axisFamily: 'time',
    isVertical: isVertical,
    scale: scale,
    range: range,
    outerScale: outerScale,
    bandScale: bandScale,
    formatters: formatters
  });
}

function buildLinearAxis(options, series, isVertical, range, outerRange) {
  var scale = options.scaleType === 'log' ? scaleLog() : scaleLinear();
  var allDatums = series.map(function (d) {
    return d.datums;
  }).flat(2);

  var _ref = options.stacked ? extent(series.map(function (s) {
    return s.datums.map(function (datum) {
      var _datum$stackData;

      return (_datum$stackData = datum.stackData) != null ? _datum$stackData : [];
    });
  }).flat(2)) : extent(allDatums, function (datum) {
    return options.getValue(datum.originalDatum);
  }),
      minValue = _ref[0],
      maxValue = _ref[1];

  var shouldNice = true;

  if (typeof options.min === 'number') {
    minValue = min([options.min, minValue]);
    shouldNice = false;
  }

  if (typeof options.max === 'number') {
    maxValue = max([options.max, maxValue]);
    shouldNice = false;
  }

  if (typeof options.minDomainLength === 'number' && !(minValue === undefined || maxValue === undefined)) {
    var mid = median([minValue, maxValue]);
    var top = mid + options.minDomainLength / 2;
    var bottom = mid - options.minDomainLength / 2;
    maxValue = Math.max(top, maxValue);
    minValue = Math.min(bottom, minValue);
  }

  if (typeof options.hardMin === 'number') {
    minValue = options.hardMin;
    shouldNice = false;
  }

  if (typeof options.hardMax === 'number') {
    maxValue = options.hardMax;
    shouldNice = false;
  }

  if (minValue === undefined || maxValue === undefined) {
    console.info({
      options: options,
      series: series,
      range: range,
      values: allDatums.map(function (d) {
        return options.getValue(d.originalDatum);
      })
    });
    throw new Error('Invalid scale min/max');
  } // Set the domain


  scale.domain([minValue, maxValue]);

  if (options.invert) {
    scale.domain(Array.from(scale.domain()).reverse());
  }

  scale.range(range);

  if (shouldNice) {
    scale.nice();
  }

  var outerScale = scale.copy().range(outerRange);
  var bandScale = options.isPrimary ? buildImpliedBandScale(options, scale, series, range) : undefined;
  var defaultFormat = scale.tickFormat();
  var formatters = {};

  var scaleFormat = function scaleFormat(value) {
    var _options$formatters$s2, _options$formatters4;

    return (_options$formatters$s2 = (_options$formatters4 = options.formatters) == null ? void 0 : _options$formatters4.scale == null ? void 0 : _options$formatters4.scale(value, _extends({}, formatters, {
      scale: undefined
    }))) != null ? _options$formatters$s2 : defaultFormat(value);
  };

  var tooltipFormat = function tooltipFormat(value) {
    var _options$formatters$t2, _options$formatters5;

    return (_options$formatters$t2 = (_options$formatters5 = options.formatters) == null ? void 0 : _options$formatters5.tooltip == null ? void 0 : _options$formatters5.tooltip(value, _extends({}, formatters, {
      tooltip: undefined
    }))) != null ? _options$formatters$t2 : scaleFormat(value);
  };

  var cursorFormat = function cursorFormat(value) {
    var _options$formatters$c2, _options$formatters6;

    return (_options$formatters$c2 = (_options$formatters6 = options.formatters) == null ? void 0 : _options$formatters6.cursor == null ? void 0 : _options$formatters6.cursor(value, _extends({}, formatters, {
      cursor: undefined
    }))) != null ? _options$formatters$c2 : tooltipFormat(value);
  };

  Object.assign(formatters, {
    "default": defaultFormat,
    scale: scaleFormat,
    tooltip: tooltipFormat,
    cursor: cursorFormat
  });
  return _extends({}, options, {
    axisFamily: 'linear',
    isVertical: isVertical,
    scale: scale,
    range: range,
    outerScale: outerScale,
    bandScale: bandScale,
    formatters: formatters
  });
}

function buildBandAxis(options, series, isVertical, range, outerRange) {
  var _options$outerBandPad2, _options$innerBandPad2;

  var domain = Array.from(new Set(series.map(function (d) {
    return d.datums;
  }).flat().map(function (datum) {
    return options.getValue(datum.originalDatum);
  })));
  var scale = scaleBand(domain, range).round(false).paddingOuter((_options$outerBandPad2 = options.outerBandPadding) != null ? _options$outerBandPad2 : 0).paddingInner((_options$innerBandPad2 = options.innerBandPadding) != null ? _options$innerBandPad2 : 0); // Invert if necessary

  if (options.invert) {
    scale.domain(Array.from(scale.domain()).reverse());
  }

  var outerScale = scale.copy().range(outerRange);

  var defaultFormat = function defaultFormat(d) {
    return d;
  };

  var formatters = {};

  var scaleFormat = function scaleFormat(value) {
    var _options$formatters$s3, _options$formatters7;

    return (_options$formatters$s3 = (_options$formatters7 = options.formatters) == null ? void 0 : _options$formatters7.scale == null ? void 0 : _options$formatters7.scale(value, _extends({}, formatters, {
      scale: undefined
    }))) != null ? _options$formatters$s3 : defaultFormat(value);
  };

  var tooltipFormat = function tooltipFormat(value) {
    var _options$formatters$t3, _options$formatters8;

    return (_options$formatters$t3 = (_options$formatters8 = options.formatters) == null ? void 0 : _options$formatters8.tooltip == null ? void 0 : _options$formatters8.tooltip(value, _extends({}, formatters, {
      tooltip: undefined
    }))) != null ? _options$formatters$t3 : scaleFormat(value);
  };

  var cursorFormat = function cursorFormat(value) {
    var _options$formatters$c3, _options$formatters9;

    return (_options$formatters$c3 = (_options$formatters9 = options.formatters) == null ? void 0 : _options$formatters9.cursor == null ? void 0 : _options$formatters9.cursor(value, _extends({}, formatters, {
      cursor: undefined
    }))) != null ? _options$formatters$c3 : tooltipFormat(value);
  };

  Object.assign(formatters, {
    "default": defaultFormat,
    scale: scaleFormat,
    tooltip: tooltipFormat,
    cursor: cursorFormat
  });
  return _extends({}, options, {
    axisFamily: 'band',
    isVertical: isVertical,
    scale: scale,
    range: range,
    outerScale: outerScale,
    formatters: formatters
  });
} //


function buildImpliedBandScale(options, scale, series, range) {
  var _options$outerBandPad3, _options$innerBandPad3;

  // Find the two closest points along axis
  var impliedBandWidth = Math.max.apply(Math, range);
  series.forEach(function (serie) {
    serie.datums.forEach(function (d1) {
      var _options$getValue;

      var one = scale((_options$getValue = options.getValue(d1.originalDatum)) != null ? _options$getValue : NaN);
      serie.datums.forEach(function (d2) {
        var _options$getValue2;

        var two = scale((_options$getValue2 = options.getValue(d2.originalDatum)) != null ? _options$getValue2 : NaN);

        if (one === two) {
          return;
        }

        var r = [one, two].sort();
        var diff = Math.abs(r[1] - r[0]);

        if (diff < impliedBandWidth) {
          impliedBandWidth = diff;
        }
      });
    });
  });
  var bandRange = Math.max.apply(Math, range);
  var bandDomain = d3Range(bandRange / impliedBandWidth);
  var bandScale = scaleBand(bandDomain, range).round(false).paddingOuter((_options$outerBandPad3 = options.outerBandPadding) != null ? _options$outerBandPad3 : 0).paddingInner((_options$innerBandPad3 = options.innerBandPadding) != null ? _options$innerBandPad3 : 0);
  return bandScale;
}