"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault")["default"];

exports.__esModule = true;
exports.Chart = Chart;

var _createForOfIteratorHelperLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/createForOfIteratorHelperLoose"));

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _d3Array = require("d3-array");

var _d3Shape = require("d3-shape");

var _jotai = require("jotai");

var _react = _interopRequireDefault(require("react"));

var _useGetLatest = _interopRequireDefault(require("../hooks/useGetLatest"));

var _useIsomorphicLayoutEffect = _interopRequireDefault(require("../hooks/useIsomorphicLayoutEffect"));

var _Area = _interopRequireDefault(require("../seriesTypes/Area"));

var _Bar = _interopRequireDefault(require("../seriesTypes/Bar"));

var _Line = _interopRequireDefault(require("../seriesTypes/Line"));

var _Utils = require("../utils/Utils");

var _buildAxis = _interopRequireDefault(require("../utils/buildAxis.linear"));

var _chartContext = require("../utils/chartContext");

var _AxisLinear = _interopRequireDefault(require("./AxisLinear"));

var _Cursors = _interopRequireDefault(require("./Cursors"));

var _Tooltip = _interopRequireDefault(require("./Tooltip"));

var _Voronoi = _interopRequireDefault(require("./Voronoi"));

var _excluded = ["options", "className", "style"];
//
//
var defaultColorScheme = ['#0f83ab', '#faa43a', '#ff4e4e', '#53cfc9', '#a2d925', '#decf3f', '#734fe9', '#cd82ad', '#006d92', '#de7c00', '#f33232', '#3f9a80', '#53c200', '#d7af00', '#4c26c9', '#d44d99'];

function defaultChartOptions(options) {
  var _options$initialWidth, _options$initialHeigh, _options$getSeriesSty, _options$getDatumStyl, _options$getSeriesOrd, _options$groupingMode, _options$showVoronoi, _options$defaultColor, _options$useIntersect, _options$intersection, _options$tooltip, _options$primaryCurso, _options$secondaryCur;

  return (0, _extends2["default"])({}, options, {
    initialWidth: (_options$initialWidth = options.initialWidth) != null ? _options$initialWidth : 300,
    initialHeight: (_options$initialHeigh = options.initialHeight) != null ? _options$initialHeigh : 200,
    getSeriesStyle: (_options$getSeriesSty = options.getSeriesStyle) != null ? _options$getSeriesSty : function () {
      return {};
    },
    getDatumStyle: (_options$getDatumStyl = options.getDatumStyle) != null ? _options$getDatumStyl : function () {
      return {};
    },
    getSeriesOrder: (_options$getSeriesOrd = options.getSeriesOrder) != null ? _options$getSeriesOrd : function (series) {
      return series;
    },
    groupingMode: (_options$groupingMode = options.groupingMode) != null ? _options$groupingMode : 'primary',
    showVoronoi: (_options$showVoronoi = options.showVoronoi) != null ? _options$showVoronoi : false,
    defaultColors: (_options$defaultColor = options.defaultColors) != null ? _options$defaultColor : defaultColorScheme,
    useIntersectionObserver: (_options$useIntersect = options.useIntersectionObserver) != null ? _options$useIntersect : true,
    intersectionObserverRootMargin: (_options$intersection = options.intersectionObserverRootMargin) != null ? _options$intersection : '1000px',
    tooltip: (_options$tooltip = options.tooltip) != null ? _options$tooltip : true,
    primaryCursor: (_options$primaryCurso = options.primaryCursor) != null ? _options$primaryCurso : true,
    secondaryCursor: (_options$secondaryCur = options.secondaryCursor) != null ? _options$secondaryCur : true
  });
}

function Chart(_ref) {
  var userOptions = _ref.options,
      className = _ref.className,
      _ref$style = _ref.style,
      style = _ref$style === void 0 ? {} : _ref$style,
      rest = (0, _objectWithoutPropertiesLoose2["default"])(_ref, _excluded);
  var options = defaultChartOptions(userOptions);

  var _React$useState = _react["default"].useState(null),
      chartElement = _React$useState[0],
      setContainerElement = _React$useState[1];

  var containerEl = chartElement == null ? void 0 : chartElement.parentElement;

  var nearestScrollableParent = _react["default"].useMemo(function () {
    var run = function run(el) {
      if (!el) {
        return null;
      }

      var grandParent = el.parentElement;

      if (!grandParent) {
        return null;
      }

      if (grandParent.scrollHeight > grandParent.clientHeight) {
        var _window$getComputedSt = window.getComputedStyle(grandParent),
            overflow = _window$getComputedSt.overflow;

        if (overflow.includes('scroll') || overflow.includes('auto')) {
          return grandParent;
        }
      }

      return run(grandParent);
    };

    return run(containerEl);
  }, [containerEl]);

  var _React$useState2 = _react["default"].useState({
    width: options.initialWidth,
    height: options.initialHeight
  }),
      _React$useState2$ = _React$useState2[0],
      width = _React$useState2$.width,
      height = _React$useState2$.height,
      setDims = _React$useState2[1];

  (0, _useIsomorphicLayoutEffect["default"])(function () {
    if (containerEl) {
      var computed = window.getComputedStyle(containerEl);

      if (!['relative', 'absolute', 'fixed'].includes(computed.position)) {
        containerEl.style.position = 'relative';
      }
    }
  }, [containerEl]);

  _react["default"].useEffect(function () {
    if (!containerEl) {
      return;
    }

    var observer = new ResizeObserver(function () {
      var rect = containerEl == null ? void 0 : containerEl.getBoundingClientRect();

      if (rect) {
        setDims({
          width: rect.width,
          height: rect.height
        });
      }
    });
    observer.observe(containerEl);
    return function () {
      observer.unobserve(containerEl);
    };
  }, [containerEl]);

  var _React$useState3 = _react["default"].useState(true),
      isIntersecting = _React$useState3[0],
      setIsIntersecting = _React$useState3[1];

  _react["default"].useEffect(function () {
    if (!containerEl || !options.useIntersectionObserver) return;
    var observer = new IntersectionObserver(function (entries) {
      for (var _iterator = (0, _createForOfIteratorHelperLoose2["default"])(entries), _step; !(_step = _iterator()).done;) {
        var entry = _step.value;

        if (entry.isIntersecting) {
          setIsIntersecting(true);
        } else {
          setIsIntersecting(false);
        }
      }
    }, {
      root: nearestScrollableParent,
      rootMargin: options.intersectionObserverRootMargin
    });
    observer.observe(containerEl);
    return function () {
      observer.unobserve(containerEl);
    };
  }, [containerEl, nearestScrollableParent, options.intersectionObserverRootMargin, options.useIntersectionObserver]);

  return /*#__PURE__*/_react["default"].createElement("div", (0, _extends2["default"])({
    ref: setContainerElement
  }, rest, {
    className: "ReactChart " + (className || ''),
    style: (0, _extends2["default"])({}, style, {
      position: 'absolute',
      width: width,
      height: height
    })
  }), isIntersecting ? /*#__PURE__*/_react["default"].createElement(ChartInner, {
    options: options,
    width: width,
    height: height
  }) : null);
}

function ChartInner(_ref2) {
  var _options$renderSVG;

  var options = _ref2.options,
      width = _ref2.width,
      height = _ref2.height;

  if (!options.primaryAxis) {
    throw new Error('A primaryAxis is required');
  }

  if (!options.secondaryAxes.length) {
    throw new Error('At least one secondaryAxis is required');
  }

  var svgRef = _react["default"].useRef(null);

  var getOptions = (0, _useGetLatest["default"])(options);

  var axisDimensionsAtom = _react["default"].useMemo(function () {
    return (0, _jotai.atom)({
      left: {},
      right: {},
      top: {},
      bottom: {}
    });
  }, []);

  var focusedDatumAtom = _react["default"].useMemo(function () {
    return (0, _jotai.atom)(null);
  }, []);

  var useAxisDimensionsAtom = _react["default"].useCallback(function () {
    // eslint-disable-next-line
    return (0, _jotai.useAtom)(axisDimensionsAtom);
  }, [axisDimensionsAtom]);

  var useFocusedDatumAtom = _react["default"].useCallback(function () {
    // eslint-disable-next-line
    return (0, _jotai.useAtom)(focusedDatumAtom);
  }, [focusedDatumAtom]); // useAtom<Datum<TDatum> | null>(focusedDatumAtom)


  var _useAxisDimensionsAto = useAxisDimensionsAtom(),
      axisDimensions = _useAxisDimensionsAto[0];

  var _useFocusedDatumAtom = useFocusedDatumAtom(),
      focusedDatum = _useFocusedDatumAtom[0];

  var gridDimensions = _react["default"].useMemo(function () {
    // Left
    var _map = ['width', 'top', 'bottom'].map(function (prop) {
      return sumAllDimensionProperties(axisDimensions.left, prop);
    }),
        axesLeftWidth = _map[0],
        axesLeftTop = _map[1],
        axesLeftBottom = _map[2];

    var _map2 = ['width', 'top', 'bottom'].map(function (prop) {
      return sumAllDimensionProperties(axisDimensions.right, prop);
    }),
        axesRightWidth = _map2[0],
        axesRightTop = _map2[1],
        axesRightBottom = _map2[2];

    var _map3 = ['height', 'left', 'right'].map(function (prop) {
      return sumAllDimensionProperties(axisDimensions.top, prop);
    }),
        axesTopHeight = _map3[0],
        axesTopLeft = _map3[1],
        axesTopRight = _map3[2];

    var _map4 = ['height', 'left', 'right'].map(function (prop) {
      return sumAllDimensionProperties(axisDimensions.bottom, prop);
    }),
        axesBottomHeight = _map4[0],
        axesBottomLeft = _map4[1],
        axesBottomRight = _map4[2];

    var gridX = Math.max(axesLeftWidth, axesTopLeft, axesBottomLeft);
    var gridY = Math.max(axesTopHeight, axesLeftTop, axesRightTop);
    var gridWidth = Math.max(0, width - Math.max(axesLeftWidth, axesTopLeft, axesBottomLeft) - Math.max(axesRightWidth, axesTopRight, axesBottomRight));
    var gridHeight = Math.max(0, height - Math.max(axesTopHeight, axesLeftTop, axesRightTop) - Math.max(axesBottomHeight, axesLeftBottom, axesRightBottom));
    return {
      gridX: gridX,
      gridY: gridY,
      gridWidth: gridWidth,
      gridHeight: gridHeight
    };
  }, [width, height, axisDimensions]);

  var series = _react["default"].useMemo(function () {
    var series = [];

    for (var seriesIndex = 0; seriesIndex < options.data.length; seriesIndex++) {
      var _originalSeries$id, _originalSeries$label;

      var originalSeries = options.data[seriesIndex];
      var seriesId = (_originalSeries$id = originalSeries.id) != null ? _originalSeries$id : seriesIndex + '';
      var seriesLabel = (_originalSeries$label = originalSeries.label) != null ? _originalSeries$label : "Series " + (seriesIndex + 1);
      var secondaryAxisId = originalSeries.secondaryAxisId;
      var originalDatums = originalSeries.data;
      var datums = [];

      for (var datumIndex = 0; datumIndex < originalDatums.length; datumIndex++) {
        var originalDatum = originalDatums[datumIndex];
        datums[datumIndex] = {
          originalSeries: originalSeries,
          seriesIndex: seriesIndex,
          seriesId: seriesId,
          seriesLabel: seriesLabel,
          secondaryAxisId: secondaryAxisId,
          index: datumIndex,
          originalDatum: originalDatum
        };
      }

      series[seriesIndex] = {
        originalSeries: originalSeries,
        index: seriesIndex,
        id: seriesId,
        label: seriesLabel,
        secondaryAxisId: secondaryAxisId,
        datums: datums
      };
    }

    if (options.secondaryAxes.some(function (axisOptions) {
      return axisOptions.stacked;
    })) {
      options.secondaryAxes.filter(function (d) {
        return d.stacked;
      }).forEach(function (secondaryAxis) {
        var _secondaryAxis$stackO;

        var axisSeries = series.filter(function (s) {
          return s.secondaryAxisId === secondaryAxis.id;
        });
        var seriesIndices = Object.keys(axisSeries);
        var stacker = (0, _d3Shape.stack)().keys(seriesIndices).value(function (_, seriesIndex, index) {
          var val = secondaryAxis.getValue(axisSeries[Number(seriesIndex)].datums[index].originalDatum);

          if (typeof val === 'undefined' || val === null) {
            return 0;
          }

          return val;
        }).offset((_secondaryAxis$stackO = secondaryAxis.stackOffset) != null ? _secondaryAxis$stackO : _d3Shape.stackOffsetNone);
        var stacked = stacker(Array.from({
          length: axisSeries.sort(function (a, b) {
            return b.datums.length - a.datums.length;
          })[0].datums.length
        }));
        stacked.forEach(function (s, sIndex) {
          s.forEach(function (datum, i) {
            // @ts-ignore
            datum.data = axisSeries[sIndex].datums[i];
            axisSeries[sIndex].datums[i].stackData = datum;
          });
        });
      });
    }

    return series;
  }, [options.data, options.secondaryAxes]);

  var primaryAxis = _react["default"].useMemo(function () {
    return (0, _buildAxis["default"])(options.primaryAxis, series, gridDimensions, width, height);
  }, [gridDimensions, height, options.primaryAxis, series, width]);

  var secondaryAxes = _react["default"].useMemo(function () {
    return options.secondaryAxes.map(function (secondaryAxis) {
      return (0, _buildAxis["default"])(secondaryAxis, series, gridDimensions, width, height);
    });
  }, [gridDimensions, height, options.secondaryAxes, series, width]);

  var axesInfo = _react["default"].useMemo(function () {
    // Make sure we're mapping x and y to the correct axes
    var xKey = primaryAxis.isVertical ? 'secondary' : 'primary';
    var yKey = primaryAxis.isVertical ? 'primary' : 'secondary';
    return {
      xKey: xKey,
      yKey: yKey
    };
  }, [primaryAxis]);

  var groupedDatums = _react["default"].useMemo(function () {
    var groupedDatums = new Map();
    var allDatums = series.map(function (s) {
      return s.datums;
    }).flat(2);
    allDatums.forEach(function (datum) {
      var primaryValue = "" + primaryAxis.getValue(datum.originalDatum);

      if (!groupedDatums.has(primaryValue)) {
        groupedDatums.set(primaryValue, []);
      }

      groupedDatums.get(primaryValue).push(datum);
    });
    allDatums.forEach(function (datum) {
      var primaryValue = "" + primaryAxis.getValue(datum.originalDatum);
      datum.group = groupedDatums.get(primaryValue);
    });
    return groupedDatums;
  }, [primaryAxis, series]);

  var getSeriesStatusStyle = _react["default"].useCallback(function (series, focusedDatum) {
    var base = {
      color: getOptions().defaultColors[series.index % (getOptions().defaultColors.length - 1)]
    };
    var status = (0, _Utils.getSeriesStatus)(series, focusedDatum);
    var statusStyles = getOptions().getSeriesStyle(series, status);
    series.style = (0, _Utils.materializeStyles)(statusStyles, base);
    return series.style;
  }, [getOptions]);

  var getDatumStatusStyle = _react["default"].useCallback(function (datum, focusedDatum) {
    var base = (0, _extends2["default"])({}, series[datum.seriesIndex].style, {
      color: getOptions().defaultColors[datum.seriesIndex % (getOptions().defaultColors.length - 1)]
    });
    var status = (0, _Utils.getDatumStatus)(datum, focusedDatum);
    var statusStyles = getOptions().getDatumStyle(datum, status);
    datum.style = (0, _Utils.materializeStyles)(statusStyles, base);
    return datum.style;
  }, [getOptions, series]); // const mouseMoveRafRef = React.useRef<number | null>()
  // const onMouseMove = (
  //   e: React.MouseEvent<SVGSVGElement, MouseEvent> | MouseEvent
  // ) => {
  //   if (mouseMoveRafRef.current) {
  //     Raf.cancel(mouseMoveRafRef.current)
  //   }
  //   mouseMoveRafRef.current = Raf(() => {
  //     mouseMoveRafRef.current = null
  //     const { clientX, clientY } = e
  //     setPointer(old => {
  //       const x = clientX - svgRect.left - gridDimensions.gridX
  //       const y = clientY - svgRect.top - gridDimensions.gridY
  //       return {
  //         ...old,
  //         svgHovered: true,
  //         x,
  //         y,
  //       }
  //     })
  //   })
  // }
  // const onMouseUp = () => {
  //   document.removeEventListener('mouseup', onMouseUp)
  //   document.removeEventListener('mousemove', onMouseMove)
  //   // if (options.brush?.onSelect && pointer.dragging) {
  //   //   if (Math.abs(pointer.startX - pointer.x) >= 20) {
  //   //     options.brush.onSelect({
  //   //       pointer,
  //   //       start: (axesInfo.primaryAxes[0].scale as ScaleLinear<
  //   //         number,
  //   //         number
  //   //       >).invert(pointer.startX),
  //   //       end: (axesInfo.primaryAxes[0].scale as ScaleLinear<
  //   //         number,
  //   //         number
  //   //       >).invert(pointer.x),
  //   //     })
  //   //   }
  //   // }
  //   setPointer(
  //     (old): Pointer => {
  //       return {
  //         ...old,
  //         dragging: false,
  //       }
  //     }
  //   )
  // }
  // const onMouseDown = () => {
  //   document.addEventListener('mouseup', onMouseUp)
  //   document.addEventListener('mousemove', onMouseMove)
  //   setPointer(
  //     (old): Pointer => {
  //       return {
  //         ...old,
  //         startX: old.x,
  //         startY: old.y,
  //         dragging: true,
  //       }
  //     }
  //   )
  // }
  // Reverse the stack order for proper z-indexing


  var reversedSeries = [].concat(series).reverse();
  var orderedSeries = options.getSeriesOrder(reversedSeries); // const focusedSeriesIndex = focusedDatum
  //   ? orderedSeries.findIndex(series => series.id === focusedDatum.seriesId)
  //   : -1
  // Bring focused series to the front
  // orderedSeries = focusedDatum
  //   ? [
  //       ...orderedSeries.slice(0, focusedSeriesIndex),
  //       ...orderedSeries.slice(focusedSeriesIndex + 1),
  //       orderedSeries[focusedSeriesIndex],
  //     ]
  //   : orderedSeries

  (0, _useIsomorphicLayoutEffect["default"])(function () {
    if (svgRef.current && svgRef.current.parentElement && !svgRef.current.parentElement.style.position) {
      svgRef.current.parentElement.style.position = 'relative';
    }
  });
  var contextValue = {
    getOptions: getOptions,
    gridDimensions: gridDimensions,
    primaryAxis: primaryAxis,
    secondaryAxes: secondaryAxes,
    axesInfo: axesInfo,
    series: series,
    orderedSeries: orderedSeries,
    groupedDatums: groupedDatums,
    width: width,
    height: height,
    getSeriesStatusStyle: getSeriesStatusStyle,
    getDatumStatusStyle: getDatumStatusStyle,
    useAxisDimensionsAtom: useAxisDimensionsAtom,
    useFocusedDatumAtom: useFocusedDatumAtom,
    svgRef: svgRef
  };
  var seriesByAxisId = (0, _d3Array.sort)((0, _d3Array.groups)(orderedSeries, function (d) {
    return d.secondaryAxisId;
  }), function (_ref3) {
    var key = _ref3[0];
    return secondaryAxes.findIndex(function (axis) {
      return axis.id === key;
    });
  });
  return /*#__PURE__*/_react["default"].createElement(_chartContext.ChartContextProvider, {
    value: (0, _useGetLatest["default"])(contextValue)
  }, /*#__PURE__*/_react["default"].createElement("div", {
    style: {
      fontFamily: 'sans-serif'
    }
  }, /*#__PURE__*/_react["default"].createElement("svg", {
    ref: svgRef,
    style: {
      width: width,
      height: height,
      overflow: options.brush ? 'hidden' : 'visible'
    } // onMouseEnter={e => {
    //   e.persist()
    //   onMouseMove(e)
    // }}
    // onMouseMove={e => {
    //   e.persist()
    //   onMouseMove(e)
    // }}
    // onMouseLeave={e => {
    //   e.persist()
    //   setPointer(old => {
    //     return {
    //       ...old,
    //       svgHovered: false,
    //     }
    //   })
    // }}
    // onMouseDown={e => {
    //   e.persist()
    //   onMouseDown()
    // }}
    ,
    onClick: function onClick(e) {
      return options.onClickDatum == null ? void 0 : options.onClickDatum(focusedDatum, e);
    }
  }, /*#__PURE__*/_react["default"].createElement("g", {
    className: "Series",
    style: {
      pointerEvents: 'none'
    }
  }, seriesByAxisId.map(function (_ref4) {
    var axisId = _ref4[0],
        series = _ref4[1];
    var secondaryAxis = secondaryAxes.find(function (d) {
      return d.id === axisId;
    });

    if (!secondaryAxis) {
      return null;
    }

    var elementType = secondaryAxis.elementType;

    var Component = function () {
      if (elementType === 'line') {
        return _Line["default"];
      }

      if (elementType === 'bar') {
        return _Bar["default"];
      }

      if (elementType === 'area') {
        return _Area["default"];
      }

      throw new Error('Invalid elementType');
    }();

    return /*#__PURE__*/_react["default"].createElement(Component, {
      key: axisId != null ? axisId : '__default__',
      primaryAxis: primaryAxis,
      secondaryAxis: secondaryAxis,
      series: series
    });
  })), /*#__PURE__*/_react["default"].createElement("g", {
    className: "axes"
  }, [primaryAxis].concat(secondaryAxes).map(function (axis) {
    return /*#__PURE__*/_react["default"].createElement(_AxisLinear["default"], (0, _extends2["default"])({
      key: [axis.position, axis.id].join('')
    }, axis));
  })), /*#__PURE__*/_react["default"].createElement(_Voronoi["default"], null), (_options$renderSVG = options.renderSVG == null ? void 0 : options.renderSVG()) != null ? _options$renderSVG : null), /*#__PURE__*/_react["default"].createElement(_Cursors["default"], null), /*#__PURE__*/_react["default"].createElement(_Tooltip["default"], null)));
}

function sumAllDimensionProperties(axisDimensions, side) {
  var sum = 0;
  Object.keys(axisDimensions).forEach(function (axisId) {
    var _axisDimensions$axisI;

    sum += ((_axisDimensions$axisI = axisDimensions[axisId]) == null ? void 0 : _axisDimensions$axisI[side]) || 0;
  });
  return sum;
}